import * as React from 'react';
import { fromSize, ensureEnum, brushToString, stringToBrush, ensureBool, initializePropertiesFromCss, NamePatcher, isValidProp, toSpinal, getModifiedProps } from "igniteui-react-core";
import { FontInfo } from "igniteui-react-core";
import { ReactRenderer, PortalManager } from "igniteui-react-core";
import { delegateCombine, delegateRemove } from "igniteui-react-core";
import { ContentChildrenManager } from "igniteui-react-core";
import { CollectionAdapter } from "igniteui-react-core";
import { NotifyCollectionChangedAction } from "igniteui-react-core";
import { ToolPanel } from './ToolPanel';
import { BaseControlTheme_$type } from "igniteui-react-core";
import { ToolPanelOrientation_$type } from './ToolPanelOrientation';
import { ToolAction } from "./ToolAction";
import { IgrToolActionCollection } from "./igr-tool-action-collection";
import { IgrToolPanelContentRefreshedEventArgs } from "./igr-tool-panel-content-refreshed-event-args";
import { IgrToolCommandEventArgs } from './igr-tool-command-event-args';
import { NestedActionMode_$type } from './NestedActionMode';
export class IgrToolPanel extends React.Component {
    set height(value) {
        this._height = value;
        if (this._elRef) {
            this._elRef.style.height = value;
            this.i.notifySizeChanged();
        }
    }
    get height() {
        return this._height;
    }
    set width(value) {
        this._width = value;
        if (this._elRef) {
            this._elRef.style.height = value;
            this.i.notifySizeChanged();
        }
    }
    get width() {
        return this._width;
    }
    _getMainRef(ref) {
        this._elRef = ref;
        this.verifyReferences();
    }
    _updateContentChildren() {
        this.contentActions.length = 0;
        let contentChildrenActual = this._contentChildrenManager.contentChildrenActual;
        for (let i = 0; i < contentChildrenActual.length; i++) {
            if ((ToolAction.$type).isAssignableFrom(contentChildrenActual[i].i.$type)) {
                this.contentActions.push(contentChildrenActual[i]);
            }
        }
        if (this._actionsAdapter !== null) {
            this._actionsAdapter.notifyContentChanged();
        }
    }
    /**
     * A collection of manually added style mappings for the treemap.
    */
    get actions() {
        if (this._actions === null) {
            let coll = new IgrToolActionCollection();
            let inner = coll._innerColl;
            inner.addListener((sender, e) => {
                switch (e.action) {
                    case NotifyCollectionChangedAction.Add:
                        this._actionsAdapter.insertManualItem(e.newStartingIndex, e.newItems.item(0));
                        break;
                    case NotifyCollectionChangedAction.Remove:
                        this._actionsAdapter.removeManualItemAt(e.oldStartingIndex);
                        break;
                    case NotifyCollectionChangedAction.Replace:
                        this._actionsAdapter.removeManualItemAt(e.oldStartingIndex);
                        this._actionsAdapter.insertManualItem(e.newStartingIndex, e.newItems.item(0));
                        break;
                    case NotifyCollectionChangedAction.Reset:
                        this._actionsAdapter.clearManualItems();
                        break;
                }
            });
            this._actions = coll;
        }
        return this._actions;
    }
    render() {
        let children = this._contentChildrenManager.getChildren(this.props.children);
        if (this._portalManager)
            this._portalManager.onRender(children);
        let div = React.createElement("div", {
            ref: this._getMainRef,
            className: "ig-tool-panel igr-tool-panel",
            children: children
        });
        return div;
    }
    verifyReferences() {
        if (this._elRef) {
            this.requestRender = this.requestRender.bind(this);
            this._portalManager = new PortalManager("toolPanelContent", this.requestRender);
            this._reactRenderer = new ReactRenderer(this._elRef, document, true, {}, this._portalManager);
            if (document) {
                this._elRef.style.display = "block";
                this._elRef.style.width = "100%";
                this._elRef.style.height = "100%";
            }
            // render needs to fire again after the portalManager is created. 
            this.requestRender();
            this.i.provideContainer(this._reactRenderer);
            this.i.notifySizeChanged();
            this._reactRenderer.addSizeWatcher(() => {
                this.i.notifySizeChanged();
            });
        }
    }
    requestRender() {
        if (this._initialized)
            this.setState({});
    }
    constructor(props) {
        super(props);
        this._implementation = null;
        this._reactRenderer = null;
        this.contentActions = [];
        /**
        * The style mappings actually present in the treemap. Do not directly modify this array.
        * This array's contents can be modified by causing React to reproject the child content.
        * Or adding and removing ranges from the manual ranges collection on the ranges property.
        */
        this.actualActions = [];
        this._actions = null;
        this._actionsAdapter = null;
        this.__p = null;
        this._hasUserValues = new Set();
        this._stylingContainer = null;
        this._stylingParent = null;
        this._inStyling = false;
        this._contentRefreshed = null;
        this._contentRefreshed_wrapped = null;
        this._onCommand = null;
        this._onCommand_wrapped = null;
        if (this._styling) {
            NamePatcher.ensureStylablePatched(Object.getPrototypeOf(this));
        }
        this._implementation = this.createImplementation();
        this._getMainRef = this._getMainRef.bind(this);
        this._contentChildrenManager = new ContentChildrenManager((ch) => ch.key || ch.props.name, (ch) => ch.key || ch.props.name, () => this._updateContentChildren());
        this._actionsAdapter = new CollectionAdapter(this.contentActions, this.i.actions, this.actualActions, (c) => c.i, (i) => {
            if (this._reactRenderer && this._elRef) {
                i._styling(this._elRef, this, this);
            }
        }, (i) => { });
        //var multiColumnComboBox = this.i;
        //multiColumnComboBox.notifySizeChanged();
        if (props) {
            this.initializeProperties();
        }
        this._initialized = true;
    }
    shouldComponentUpdate(nextProps, nextState) {
        const mod = getModifiedProps(this.props, nextProps);
        for (const p of Object.keys(mod)) {
            if (isValidProp(this, p)) {
                this[p] = mod[p];
            }
        }
        return true;
    }
    initializeProperties() {
        for (const p of Object.keys(this.props)) {
            if (isValidProp(this, p)) {
                this[p] = this.props[p];
            }
        }
    }
    // supports angular themes or custom properties set in CSS
    updateStyle() {
        this._styling(this._elRef, this);
    }
    componentWillUnmount() {
        this.i.destroy();
        if (this._reactRenderer)
            this._reactRenderer.destroy();
    }
    componentDidMount() {
        this._elRef.style.width = this._width ? this._width : "";
        this._elRef.style.height = this._height ? this._height : "";
        this.i.notifySizeChanged();
        this.initializeContent();
    }
    initializeContent() {
        this._styling(this._elRef, this);
        if (this.actualActions && this.actualActions.length > 0) {
            var currProperty = this.actualActions;
            for (var i = 0; i < currProperty.length; i++) {
                currProperty[i]._styling(this._elRef, this, this);
            }
        }
        this.updateStyle();
        this.i.notifySizeChanged();
    }
    createImplementation() {
        return new ToolPanel();
    }
    get i() {
        return this._implementation;
    }
    /**
 * Gets or sets if the tool actions should display their submenus on mouse hover.
*/
    get showOnHover() {
        return this.i.ac;
    }
    set showOnHover(v) {
        this.i.ac = ensureBool(v);
    }
    /**
     * Gets or Sets the property name that contains the values.
    */
    get cellTextStyle() {
        if (this.i.t == null) {
            return null;
        }
        return this.i.t.fontString;
    }
    set cellTextStyle(v) {
        let fi = new FontInfo();
        fi.fontString = v;
        this.i.t = fi;
    }
    /**
     * Gets or sets the base built in theme to use for the checkbox.
    */
    get baseTheme() {
        return this.i.r;
    }
    set baseTheme(v) {
        this.i.r = ensureEnum(BaseControlTheme_$type, v);
    }
    /**
     * Gets or sets the color to use for the background of the component.
    */
    get backgroundColor() {
        return brushToString(this.i.e8);
    }
    set backgroundColor(v) {
        this.i.e8 = stringToBrush(v);
    }
    get actualBackgroundColor() {
        return brushToString(this.i.ev);
    }
    set actualBackgroundColor(v) {
        this.i.ev = stringToBrush(v);
    }
    /**
     * Gets or sets the background color for submenus.
    */
    get submenuBackgroundColor() {
        return brushToString(this.i.fh);
    }
    set submenuBackgroundColor(v) {
        this.i.fh = stringToBrush(v);
    }
    get actualSubmenuBackgroundColor() {
        return brushToString(this.i.e4);
    }
    set actualSubmenuBackgroundColor(v) {
        this.i.e4 = stringToBrush(v);
    }
    /**
     * Gets or sets the group header background color.
    */
    get groupHeaderBackgroundColor() {
        return brushToString(this.i.fa);
    }
    set groupHeaderBackgroundColor(v) {
        this.i.fa = stringToBrush(v);
    }
    get actualGroupHeaderBackgroundColor() {
        return brushToString(this.i.ex);
    }
    set actualGroupHeaderBackgroundColor(v) {
        this.i.ex = stringToBrush(v);
    }
    /**
     * Gets or sets the separator background color.
    */
    get separatorBackgroundColor() {
        return brushToString(this.i.fg);
    }
    set separatorBackgroundColor(v) {
        this.i.fg = stringToBrush(v);
    }
    get actualSeparatorBackgroundColor() {
        return brushToString(this.i.e3);
    }
    set actualSeparatorBackgroundColor(v) {
        this.i.e3 = stringToBrush(v);
    }
    /**
     * Gets or sets the group header separator background color.
    */
    get groupHeaderSeparatorBackgroundColor() {
        return brushToString(this.i.fb);
    }
    set groupHeaderSeparatorBackgroundColor(v) {
        this.i.fb = stringToBrush(v);
    }
    get actualGroupHeaderSeparatorBackgroundColor() {
        return brushToString(this.i.ey);
    }
    set actualGroupHeaderSeparatorBackgroundColor(v) {
        this.i.ey = stringToBrush(v);
    }
    /**
     * Gets or sets the hover background color.
    */
    get hoverBackgroundColor() {
        return brushToString(this.i.fd);
    }
    set hoverBackgroundColor(v) {
        this.i.fd = stringToBrush(v);
    }
    get actualHoverBackgroundColor() {
        return brushToString(this.i.e0);
    }
    set actualHoverBackgroundColor(v) {
        this.i.e0 = stringToBrush(v);
    }
    /**
     * Gets or sets the color to use for the text of the component.
    */
    get textColor() {
        return brushToString(this.i.fi);
    }
    set textColor(v) {
        this.i.fi = stringToBrush(v);
    }
    get actualTextColor() {
        return brushToString(this.i.e5);
    }
    set actualTextColor(v) {
        this.i.e5 = stringToBrush(v);
    }
    /**
     * Gets or sets the color to use for the text in disabled tool actions.
    */
    get disabledTextColor() {
        return brushToString(this.i.e9);
    }
    set disabledTextColor(v) {
        this.i.e9 = stringToBrush(v);
    }
    get actualDisabledTextColor() {
        return brushToString(this.i.ew);
    }
    set actualDisabledTextColor(v) {
        this.i.ew = stringToBrush(v);
    }
    /**
     * Gets or sets the text color for group header tool actions.
    */
    get groupHeaderTextColor() {
        return brushToString(this.i.fc);
    }
    set groupHeaderTextColor(v) {
        this.i.fc = stringToBrush(v);
    }
    get actualGroupHeaderTextColor() {
        return brushToString(this.i.ez);
    }
    set actualGroupHeaderTextColor(v) {
        this.i.ez = stringToBrush(v);
    }
    /**
     * Gets or sets the tooltip text color.
    */
    get toolTipTextColor() {
        return brushToString(this.i.fk);
    }
    set toolTipTextColor(v) {
        this.i.fk = stringToBrush(v);
    }
    get actualToolTipTextColor() {
        return brushToString(this.i.e7);
    }
    set actualToolTipTextColor(v) {
        this.i.e7 = stringToBrush(v);
    }
    /**
     * Gets or sets the tooltip background color.
    */
    get toolTipBackgroundColor() {
        return brushToString(this.i.fj);
    }
    set toolTipBackgroundColor(v) {
        this.i.fj = stringToBrush(v);
    }
    /**
     * Gets the actual tooltip background color.
    */
    get actualToolTipBackgroundColor() {
        return brushToString(this.i.e6);
    }
    set actualToolTipBackgroundColor(v) {
        this.i.e6 = stringToBrush(v);
    }
    /**
     * Gets or sets the tooltip corner radius.
    */
    get toolTipCornerRadius() {
        return this.i.bl;
    }
    set toolTipCornerRadius(v) {
        this.i.bl = +v;
    }
    /**
     * Gets the actual tooltip corner radius.
    */
    get actualToolTipCornerRadius() {
        return this.i.bb;
    }
    set actualToolTipCornerRadius(v) {
        this.i.bb = +v;
    }
    /**
     * Gets or sets the shadow elevation for the tooltip.
    */
    get toolTipElevation() {
        return this.i.bm;
    }
    set toolTipElevation(v) {
        this.i.bm = +v;
    }
    /**
     * Gets the actual shadow elevation used by the tooltip.
    */
    get actualToolTipElevation() {
        return this.i.bc;
    }
    set actualToolTipElevation(v) {
        this.i.bc = +v;
    }
    /**
     * Gets or sets the fill color of icons in the ToolPanel.
    */
    get iconFill() {
        return brushToString(this.i.fe);
    }
    set iconFill(v) {
        this.i.fe = stringToBrush(v);
    }
    /**
     * Gets the actual icon fill color.
    */
    get actualIconFill() {
        return brushToString(this.i.e1);
    }
    set actualIconFill(v) {
        this.i.e1 = stringToBrush(v);
    }
    /**
     * Gets or sets the stroke color of icons in the ToolPanel.
    */
    get iconStroke() {
        return brushToString(this.i.ff);
    }
    set iconStroke(v) {
        this.i.ff = stringToBrush(v);
    }
    /**
     * Gets the actual icon stroke color.
    */
    get actualIconStroke() {
        return brushToString(this.i.e2);
    }
    set actualIconStroke(v) {
        this.i.e2 = stringToBrush(v);
    }
    /**
     * Gets or sets the left padding for separators in the vertical orientation.
    */
    get separatorVerticalPaddingLeft() {
        return this.i.bi;
    }
    set separatorVerticalPaddingLeft(v) {
        this.i.bi = +v;
    }
    get actualSeparatorVerticalPaddingLeft() {
        return this.i.a8;
    }
    set actualSeparatorVerticalPaddingLeft(v) {
        this.i.a8 = +v;
    }
    /**
     * Gets or sets the right padding for separators in the vertical orientation.
    */
    get separatorVerticalPaddingRight() {
        return this.i.bj;
    }
    set separatorVerticalPaddingRight(v) {
        this.i.bj = +v;
    }
    get actualSeparatorVerticalPaddingRight() {
        return this.i.a9;
    }
    set actualSeparatorVerticalPaddingRight(v) {
        this.i.a9 = +v;
    }
    /**
     * Gets or sets the top padding for separators in the vertical orientation.
    */
    get separatorVerticalPaddingTop() {
        return this.i.bk;
    }
    set separatorVerticalPaddingTop(v) {
        this.i.bk = +v;
    }
    get actualSeparatorVerticalPaddingTop() {
        return this.i.ba;
    }
    set actualSeparatorVerticalPaddingTop(v) {
        this.i.ba = +v;
    }
    /**
     * Gets or sets the bottom padding for separators in the vertical orientation.
    */
    get separatorVerticalPaddingBottom() {
        return this.i.bh;
    }
    set separatorVerticalPaddingBottom(v) {
        this.i.bh = +v;
    }
    get actualSeparatorVerticalPaddingBottom() {
        return this.i.a7;
    }
    set actualSeparatorVerticalPaddingBottom(v) {
        this.i.a7 = +v;
    }
    /**
     * Gets or sets the left padding for separators in the horizontal orientation.
    */
    get separatorHorizontalPaddingLeft() {
        return this.i.be;
    }
    set separatorHorizontalPaddingLeft(v) {
        this.i.be = +v;
    }
    get actualSeparatorHorizontalPaddingLeft() {
        return this.i.a4;
    }
    set actualSeparatorHorizontalPaddingLeft(v) {
        this.i.a4 = +v;
    }
    /**
     * Gets or sets the right padding for separators in the horizontal orientation.
    */
    get separatorHorizontalPaddingRight() {
        return this.i.bf;
    }
    set separatorHorizontalPaddingRight(v) {
        this.i.bf = +v;
    }
    get actualSeparatorHorizontalPaddingRight() {
        return this.i.a5;
    }
    set actualSeparatorHorizontalPaddingRight(v) {
        this.i.a5 = +v;
    }
    /**
     * Gets or sets the top padding for separators in the horizontal orientation.
    */
    get separatorHorizontalPaddingTop() {
        return this.i.bg;
    }
    set separatorHorizontalPaddingTop(v) {
        this.i.bg = +v;
    }
    get actualSeparatorHorizontalPaddingTop() {
        return this.i.a6;
    }
    set actualSeparatorHorizontalPaddingTop(v) {
        this.i.a6 = +v;
    }
    /**
     * Gets or sets the bottom padding for separators in the horizontal orientation.
    */
    get separatorHorizontalPaddingBottom() {
        return this.i.bd;
    }
    set separatorHorizontalPaddingBottom(v) {
        this.i.bd = +v;
    }
    get actualSeparatorHorizontalPaddingBottom() {
        return this.i.a3;
    }
    set actualSeparatorHorizontalPaddingBottom(v) {
        this.i.a3 = +v;
    }
    /**
     * Gets or sets the color to use for the background of the component.
    */
    get orientation() {
        return this.i.o;
    }
    set orientation(v) {
        this.i.o = ensureEnum(ToolPanelOrientation_$type, v);
    }
    /**
     * Gets or sets the row height for the panel.
    */
    get rowHeight() {
        return this.i.bn;
    }
    set rowHeight(v) {
        this.i.bn = +v;
    }
    /**
     * Gets the actual row height used by the panel.
    */
    get actualRowHeight() {
        return this.i.a2;
    }
    /**
     * Gets or sets the mode used when displaying nested actions.
    */
    get nestedActionMode() {
        return this.i.e;
    }
    set nestedActionMode(v) {
        this.i.e = ensureEnum(NestedActionMode_$type, v);
    }
    findByName(name) {
        if (this.findEphemera) {
            if (name && name.indexOf("@@e:") == 0) {
                return this.findEphemera(name);
            }
        }
        if (this.actions != null && this.actions.findByName && this.actions.findByName(name)) {
            return this.actions.findByName(name);
        }
        return null;
    }
    get hasUserValues() {
        return this._hasUserValues;
    }
    __m(propertyName) {
        if (!this._inStyling) {
            this._hasUserValues.add(propertyName);
        }
    }
    _styling(container, component, parent) {
        if (this._inStyling) {
            return;
        }
        this._inStyling = true;
        this._stylingContainer = container;
        this._stylingParent = component;
        let genericPrefix = "";
        let typeName = this.i.$type.name;
        if (typeName.indexOf("Xam") === 0) {
            typeName = typeName.substring(3);
        }
        genericPrefix = toSpinal("ToolPanel");
        let additionalPrefixes = [];
        let prefix = toSpinal(typeName);
        additionalPrefixes.push(prefix + "-");
        let b = this.i.$type.baseType;
        while (b && b.name != "Object" &&
            b.name != "Base" &&
            b.name != "Control" &&
            b.Name != "DependencyObject" &&
            b.Name != "FrameworkElement") {
            typeName = b.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            let basePrefix = toSpinal(typeName);
            additionalPrefixes.push(basePrefix + "-");
            b = b.baseType;
        }
        if (parent) {
            let parentTypeName = parent.i.$type.name;
            if (parentTypeName.indexOf("Xam") === 0) {
                parentTypeName = parentTypeName.substring(3);
            }
            let parentPrefix = toSpinal(parentTypeName);
            additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
            additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
        }
        initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
        if (this._otherStyling) {
            this._otherStyling(container, component, parent);
        }
        this._inStyling = false;
    }
    refresh() {
        this.i.dx();
    }
    getDesiredSize() {
        let iv = this.i.fl();
        return fromSize(iv);
    }
    /**
     * Forces any open sub menus to close.
    
    */
    closeSubmenus() {
        this.i.dd();
    }
    get contentRefreshed() {
        return this._contentRefreshed;
    }
    set contentRefreshed(ev) {
        if (this._contentRefreshed_wrapped !== null) {
            this.i.contentRefreshed = delegateRemove(this.i.contentRefreshed, this._contentRefreshed_wrapped);
            this._contentRefreshed_wrapped = null;
            this._contentRefreshed = null;
        }
        this._contentRefreshed = ev;
        this._contentRefreshed_wrapped = (o, e) => {
            let outerArgs = new IgrToolPanelContentRefreshedEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforeContentRefreshed) {
                this.beforeContentRefreshed(this, outerArgs);
            }
            if (this._contentRefreshed) {
                this._contentRefreshed(this, outerArgs);
            }
        };
        this.i.contentRefreshed = delegateCombine(this.i.contentRefreshed, this._contentRefreshed_wrapped);
        ;
    }
    get onCommand() {
        return this._onCommand;
    }
    set onCommand(ev) {
        if (this._onCommand_wrapped !== null) {
            this.i.onCommand = delegateRemove(this.i.onCommand, this._onCommand_wrapped);
            this._onCommand_wrapped = null;
            this._onCommand = null;
        }
        this._onCommand = ev;
        this._onCommand_wrapped = (o, e) => {
            let outerArgs = new IgrToolCommandEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforeOnCommand) {
                this.beforeOnCommand(this, outerArgs);
            }
            if (this._onCommand) {
                this._onCommand(this, outerArgs);
            }
        };
        this.i.onCommand = delegateCombine(this.i.onCommand, this._onCommand_wrapped);
        ;
    }
}
