import * as React from 'react';
import { ensureEnum, brushToString, stringToBrush, ensureBool, initializePropertiesFromCss, NamePatcher, isValidProp, toSpinal, getModifiedProps } from "igniteui-react-core";
import { FontInfo } from "igniteui-react-core";
import { ReactRenderer, PortalManager } from "igniteui-react-core";
import { delegateCombine, delegateRemove } from "igniteui-react-core";
import { ContentChildrenManager } from "igniteui-react-core";
import { CollectionAdapter } from "igniteui-react-core";
import { NotifyCollectionChangedAction } from "igniteui-react-core";
import { Toolbar } from './Toolbar';
import { BaseControlTheme_$type } from "igniteui-react-core";
import { SyncableObservableCollection$1 } from "igniteui-react-core";
import { ToolbarOrientation_$type } from './ToolbarOrientation';
import { ToolActionCollection as ToolActionCollection_internal } from "./ToolActionCollection";
import { ToolAction } from "./ToolAction";
import { IgrToolActionCollection } from "./igr-tool-action-collection";
import { IgrToolCommandEventArgs } from './igr-tool-command-event-args';
export class IgrToolbar extends React.Component {
    set height(value) {
        this._height = value;
        this.i.userSetHeight = value ? true : false;
        if (this._elRef) {
            this._elRef.style.height = value;
            this.i.notifySizeChanged();
        }
    }
    get height() {
        return this._height;
    }
    set width(value) {
        this._width = value;
        this.i.userSetWidth = value ? true : false;
        if (this._elRef) {
            this._elRef.style.height = value;
            this.i.notifySizeChanged();
        }
    }
    get width() {
        return this._width;
    }
    _getMainRef(ref) {
        this._elRef = ref;
        this.verifyReferences();
    }
    _updateContentChildren() {
        this.contentActions.length = 0;
        let contentChildrenActual = this._contentChildrenManager.contentChildrenActual;
        for (let i = 0; i < contentChildrenActual.length; i++) {
            if ((ToolAction.$type).isAssignableFrom(contentChildrenActual[i].i.$type)) {
                this.contentActions.push(contentChildrenActual[i]);
            }
        }
        if (this._actionsAdapter !== null) {
            this._actionsAdapter.notifyContentChanged();
        }
    }
    /**
     * A collection of manually added style mappings for the treemap.
    */
    get actions() {
        if (this._actions === null) {
            let coll = new IgrToolActionCollection();
            let inner = coll._innerColl;
            inner.addListener((sender, e) => {
                switch (e.action) {
                    case NotifyCollectionChangedAction.Add:
                        this._actionsAdapter.insertManualItem(e.newStartingIndex, e.newItems.item(0));
                        break;
                    case NotifyCollectionChangedAction.Remove:
                        this._actionsAdapter.removeManualItemAt(e.oldStartingIndex);
                        break;
                    case NotifyCollectionChangedAction.Replace:
                        this._actionsAdapter.removeManualItemAt(e.oldStartingIndex);
                        this._actionsAdapter.insertManualItem(e.newStartingIndex, e.newItems.item(0));
                        break;
                    case NotifyCollectionChangedAction.Reset:
                        this._actionsAdapter.clearManualItems();
                        break;
                }
            });
            this._actions = coll;
        }
        return this._actions;
    }
    render() {
        let children = this._contentChildrenManager.getChildren(this.props.children);
        if (this._portalManager)
            this._portalManager.onRender(children);
        let div = React.createElement("div", {
            ref: this._getMainRef,
            className: "ig-toolbar igr-toolbar",
            children: children
        });
        return div;
    }
    verifyReferences() {
        if (this._elRef) {
            this.requestRender = this.requestRender.bind(this);
            this._portalManager = new PortalManager("toolbarContent", this.requestRender);
            this._reactRenderer = new ReactRenderer(this._elRef, document, true, {}, this._portalManager);
            if (document) {
                this._elRef.style.display = "block";
                this._elRef.style.width = "100%";
                this._elRef.style.height = "100%";
            }
            // render needs to fire again after the portalManager is created. 
            this.requestRender();
            this.i.provideContainer(this._reactRenderer);
            this.i.notifySizeChanged();
            this._reactRenderer.addSizeWatcher(() => {
                this.i.notifySizeChanged();
            });
        }
    }
    requestRender() {
        if (this._initialized)
            this.setState({});
    }
    constructor(props) {
        super(props);
        this._implementation = null;
        this._reactRenderer = null;
        this.contentActions = [];
        /**
        * The style mappings actually present in the treemap. Do not directly modify this array.
        * This array's contents can be modified by causing React to reproject the child content.
        * Or adding and removing ranges from the manual ranges collection on the ranges property.
        */
        this.combinedActions = [];
        this._actions = null;
        this._actionsAdapter = null;
        this._autoGeneratedActions = null;
        this._actualActions = null;
        this.__p = null;
        this._hasUserValues = new Set();
        this._stylingContainer = null;
        this._stylingParent = null;
        this._inStyling = false;
        this._onCommand = null;
        this._onCommand_wrapped = null;
        if (this._styling) {
            NamePatcher.ensureStylablePatched(Object.getPrototypeOf(this));
        }
        this._implementation = this.createImplementation();
        this._getMainRef = this._getMainRef.bind(this);
        this._contentChildrenManager = new ContentChildrenManager((ch) => ch.key || ch.props.name, (ch) => ch.key || ch.props.name, () => this._updateContentChildren());
        this._actionsAdapter = new CollectionAdapter(this.contentActions, this.i.actions, this.combinedActions, (c) => c.i, (i) => {
            if (this._reactRenderer && this._elRef) {
                i._styling(this._elRef, this, this);
            }
        }, (i) => { });
        //var multiColumnComboBox = this.i;
        //multiColumnComboBox.notifySizeChanged();
        if (props) {
            this.initializeProperties();
        }
        this._initialized = true;
    }
    shouldComponentUpdate(nextProps, nextState) {
        const mod = getModifiedProps(this.props, nextProps);
        for (const p of Object.keys(mod)) {
            if (isValidProp(this, p)) {
                this[p] = mod[p];
            }
        }
        return true;
    }
    initializeProperties() {
        for (const p of Object.keys(this.props)) {
            if (isValidProp(this, p)) {
                this[p] = this.props[p];
            }
        }
    }
    // supports angular themes or custom properties set in CSS
    updateStyle() {
        this._styling(this._elRef, this);
    }
    componentWillUnmount() {
        this.i.destroy();
        if (this._reactRenderer)
            this._reactRenderer.destroy();
    }
    componentDidMount() {
        this._elRef.style.width = this._width ? this._width : "";
        this._elRef.style.height = this._height ? this._height : "";
        this.i.notifySizeChanged();
        this.initializeContent();
    }
    initializeContent() {
        this._styling(this._elRef, this);
        if (this.combinedActions && this.combinedActions.length > 0) {
            var currProperty = this.combinedActions;
            for (var i = 0; i < currProperty.length; i++) {
                currProperty[i]._styling(this._elRef, this, this);
            }
        }
        this.updateStyle();
        this.i.notifySizeChanged();
    }
    createImplementation() {
        return new Toolbar();
    }
    get i() {
        return this._implementation;
    }
    /**
     * Gets or sets a target component which the toolbar can request actions from.
     */
    get target() {
        var ret = this.i.target;
        if (!ret) {
            return ret;
        }
        if (ret.externalObject != null) {
            ret = ret.externalObject;
        }
        return ret;
    }
    set target(v) {
        if (v && v._implementation) {
            v = v._implementation;
        }
        this.i.target = v;
    }
    /**
 * Gets or sets the base built in theme to use for the checkbox.
*/
    get baseTheme() {
        return this.i.q;
    }
    set baseTheme(v) {
        this.i.q = ensureEnum(BaseControlTheme_$type, v);
    }
    /**
     * Gets or sets the data or data source instance to which to bind the grid.
    */
    get autoGeneratedActions() {
        if (this._autoGeneratedActions === null) {
            let coll = new IgrToolActionCollection();
            let innerColl = this.i.autoGeneratedActions;
            if (!innerColl) {
                innerColl = new ToolActionCollection_internal();
            }
            this._autoGeneratedActions = coll._fromInner(innerColl);
            this.i.autoGeneratedActions = innerColl;
        }
        return this._autoGeneratedActions;
    }
    set autoGeneratedActions(v) {
        if (this._autoGeneratedActions !== null) {
            this._autoGeneratedActions._setSyncTarget(null);
            this._autoGeneratedActions = null;
        }
        let coll = new IgrToolActionCollection();
        this._autoGeneratedActions = coll._fromOuter(v);
        let syncColl = new SyncableObservableCollection$1(ToolAction.$type);
        let innerColl = this.i.autoGeneratedActions;
        if (!innerColl) {
            innerColl = new ToolActionCollection_internal();
        }
        syncColl._inner = innerColl;
        syncColl.clear();
        this._autoGeneratedActions._setSyncTarget(syncColl);
        this.i.autoGeneratedActions = innerColl;
    }
    /**
     * Gets or sets the data or data source instance to which to bind the grid.
    */
    get actualActions() {
        if (this._actualActions === null) {
            let coll = new IgrToolActionCollection();
            let innerColl = this.i.actualActions;
            if (!innerColl) {
                innerColl = new ToolActionCollection_internal();
            }
            this._actualActions = coll._fromInner(innerColl);
            this.i.actualActions = innerColl;
        }
        return this._actualActions;
    }
    set actualActions(v) {
        if (this._actualActions !== null) {
            this._actualActions._setSyncTarget(null);
            this._actualActions = null;
        }
        let coll = new IgrToolActionCollection();
        this._actualActions = coll._fromOuter(v);
        let syncColl = new SyncableObservableCollection$1(ToolAction.$type);
        let innerColl = this.i.actualActions;
        if (!innerColl) {
            innerColl = new ToolActionCollection_internal();
        }
        syncColl._inner = innerColl;
        syncColl.clear();
        this._actualActions._setSyncTarget(syncColl);
        this.i.actualActions = innerColl;
    }
    /**
     * Gets or sets the orientation of the toolbar.
    */
    get orientation() {
        return this.i.k;
    }
    set orientation(v) {
        this.i.k = ensureEnum(ToolbarOrientation_$type, v);
    }
    /**
     * Gets or sets the color to use for the background of the component.
    */
    get backgroundColor() {
        return brushToString(this.i.c1);
    }
    set backgroundColor(v) {
        this.i.c1 = stringToBrush(v);
    }
    /**
     * Gets or sets the background color for submenus.
    */
    get submenuBackgroundColor() {
        return brushToString(this.i.c9);
    }
    set submenuBackgroundColor(v) {
        this.i.c9 = stringToBrush(v);
    }
    /**
     * Gets or sets the group header background color.
    */
    get groupHeaderBackgroundColor() {
        return brushToString(this.i.c3);
    }
    set groupHeaderBackgroundColor(v) {
        this.i.c3 = stringToBrush(v);
    }
    /**
     * Gets or sets the separator background color.
    */
    get separatorBackgroundColor() {
        return brushToString(this.i.c8);
    }
    set separatorBackgroundColor(v) {
        this.i.c8 = stringToBrush(v);
    }
    /**
     * Gets or sets the hover background color.
    */
    get hoverBackgroundColor() {
        return brushToString(this.i.c5);
    }
    set hoverBackgroundColor(v) {
        this.i.c5 = stringToBrush(v);
    }
    /**
     * Gets or sets the color to use for the text of the component.
    */
    get textColor() {
        return brushToString(this.i.da);
    }
    set textColor(v) {
        this.i.da = stringToBrush(v);
    }
    /**
     * Gets or sets the color to use for the text in disabled tool actions.
    */
    get disabledTextColor() {
        return brushToString(this.i.c2);
    }
    set disabledTextColor(v) {
        this.i.c2 = stringToBrush(v);
    }
    /**
     * Gets or sets the text color for group header tool actions.
    */
    get groupHeaderTextColor() {
        return brushToString(this.i.c4);
    }
    set groupHeaderTextColor(v) {
        this.i.c4 = stringToBrush(v);
    }
    /**
     * Gets or sets the tooltip text color.
    */
    get toolTipTextColor() {
        return brushToString(this.i.dc);
    }
    set toolTipTextColor(v) {
        this.i.dc = stringToBrush(v);
    }
    /**
     * Gets or sets the tooltip background color.
    */
    get toolTipBackgroundColor() {
        return brushToString(this.i.db);
    }
    set toolTipBackgroundColor(v) {
        this.i.db = stringToBrush(v);
    }
    /**
     * Gets or sets the tooltip corner radius.
    */
    get toolTipCornerRadius() {
        return this.i.ay;
    }
    set toolTipCornerRadius(v) {
        this.i.ay = +v;
    }
    /**
     * Gets or sets the shadow elevation for the tooltip.
    */
    get toolTipElevation() {
        return this.i.az;
    }
    set toolTipElevation(v) {
        this.i.az = +v;
    }
    /**
     * Gets or sets the fill color of icons in the ToolPanel.
    */
    get iconFill() {
        return brushToString(this.i.c6);
    }
    set iconFill(v) {
        this.i.c6 = stringToBrush(v);
    }
    /**
     * Gets or sets the stroke color of icons in the ToolPanel.
    */
    get iconStroke() {
        return brushToString(this.i.c7);
    }
    set iconStroke(v) {
        this.i.c7 = stringToBrush(v);
    }
    /**
     * Gets or Sets the property name that contains the values.
    */
    get cellTextStyle() {
        if (this.i.t == null) {
            return null;
        }
        return this.i.t.fontString;
    }
    set cellTextStyle(v) {
        let fi = new FontInfo();
        fi.fontString = v;
        this.i.t = fi;
    }
    /**
     * Gets or sets the left padding for separators in the vertical orientation.
    */
    get separatorVerticalPaddingLeft() {
        return this.i.av;
    }
    set separatorVerticalPaddingLeft(v) {
        this.i.av = +v;
    }
    /**
     * Gets or sets the right padding for separators in the vertical orientation.
    */
    get separatorVerticalPaddingRight() {
        return this.i.aw;
    }
    set separatorVerticalPaddingRight(v) {
        this.i.aw = +v;
    }
    /**
     * Gets or sets the top padding for separators in the vertical orientation.
    */
    get separatorVerticalPaddingTop() {
        return this.i.ax;
    }
    set separatorVerticalPaddingTop(v) {
        this.i.ax = +v;
    }
    /**
     * Gets or sets the bottom padding for separators in the vertical orientation.
    */
    get separatorVerticalPaddingBottom() {
        return this.i.au;
    }
    set separatorVerticalPaddingBottom(v) {
        this.i.au = +v;
    }
    /**
     * Gets or sets the left padding for separators in the horizontal orientation.
    */
    get separatorHorizontalPaddingLeft() {
        return this.i.ar;
    }
    set separatorHorizontalPaddingLeft(v) {
        this.i.ar = +v;
    }
    /**
     * Gets or sets the right padding for separators in the horizontal orientation.
    */
    get separatorHorizontalPaddingRight() {
        return this.i.as;
    }
    set separatorHorizontalPaddingRight(v) {
        this.i.as = +v;
    }
    /**
     * Gets or sets the top padding for separators in the horizontal orientation.
    */
    get separatorHorizontalPaddingTop() {
        return this.i.at;
    }
    set separatorHorizontalPaddingTop(v) {
        this.i.at = +v;
    }
    /**
     * Gets or sets the bottom padding for separators in the horizontal orientation.
    */
    get separatorHorizontalPaddingBottom() {
        return this.i.aq;
    }
    set separatorHorizontalPaddingBottom(v) {
        this.i.aq = +v;
    }
    /**
     * Gets or sets if the tool actions should display their submenus on mouse hover.
    */
    get showOnHover() {
        return this.i.ad;
    }
    set showOnHover(v) {
        this.i.ad = ensureBool(v);
    }
    findByName(name) {
        if (this.findEphemera) {
            if (name && name.indexOf("@@e:") == 0) {
                return this.findEphemera(name);
            }
        }
        if (this.autoGeneratedActions != null && this.autoGeneratedActions.findByName && this.autoGeneratedActions.findByName(name)) {
            return this.autoGeneratedActions.findByName(name);
        }
        if (this.actions != null && this.actions.findByName && this.actions.findByName(name)) {
            return this.actions.findByName(name);
        }
        if (this.actualActions != null && this.actualActions.findByName && this.actualActions.findByName(name)) {
            return this.actualActions.findByName(name);
        }
        return null;
    }
    get hasUserValues() {
        return this._hasUserValues;
    }
    __m(propertyName) {
        if (!this._inStyling) {
            this._hasUserValues.add(propertyName);
        }
    }
    _styling(container, component, parent) {
        if (this._inStyling) {
            return;
        }
        this._inStyling = true;
        this._stylingContainer = container;
        this._stylingParent = component;
        let genericPrefix = "";
        let typeName = this.i.$type.name;
        if (typeName.indexOf("Xam") === 0) {
            typeName = typeName.substring(3);
        }
        genericPrefix = toSpinal("Toolbar");
        let additionalPrefixes = [];
        let prefix = toSpinal(typeName);
        additionalPrefixes.push(prefix + "-");
        let b = this.i.$type.baseType;
        while (b && b.name != "Object" &&
            b.name != "Base" &&
            b.name != "Control" &&
            b.Name != "DependencyObject" &&
            b.Name != "FrameworkElement") {
            typeName = b.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            let basePrefix = toSpinal(typeName);
            additionalPrefixes.push(basePrefix + "-");
            b = b.baseType;
        }
        if (parent) {
            let parentTypeName = parent.i.$type.name;
            if (parentTypeName.indexOf("Xam") === 0) {
                parentTypeName = parentTypeName.substring(3);
            }
            let parentPrefix = toSpinal(parentTypeName);
            additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
            additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
        }
        initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
        if (this._otherStyling) {
            this._otherStyling(container, component, parent);
        }
        this._inStyling = false;
    }
    /**
 * Forces any open sub menus to close.

*/
    closeSubmenus() {
        this.i.b5();
    }
    onCommandStateChanged(id, type, value) {
        this.i.onCommandStateChanged(id, (type == null ? null : type), value);
    }
    registerIconFromText(iconCollection, iconName, iconText) {
        this.i.cm(iconCollection, iconName, iconText);
    }
    getIconFromCache(iconCollection, iconName) {
        let iv = this.i.bc(iconCollection, iconName);
        return (iv);
    }
    /**
     * Event raised when a command is triggered in the toolbar.
    */
    get onCommand() {
        return this._onCommand;
    }
    set onCommand(ev) {
        if (this._onCommand_wrapped !== null) {
            this.i.onCommand = delegateRemove(this.i.onCommand, this._onCommand_wrapped);
            this._onCommand_wrapped = null;
            this._onCommand = null;
        }
        this._onCommand = ev;
        this._onCommand_wrapped = (o, e) => {
            let outerArgs = new IgrToolCommandEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforeOnCommand) {
                this.beforeOnCommand(this, outerArgs);
            }
            if (this._onCommand) {
                this._onCommand(this, outerArgs);
            }
        };
        this.i.onCommand = delegateCombine(this.i.onCommand, this._onCommand_wrapped);
        ;
    }
}
